/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.auth;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.core.common.model.notification.HmacSecret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.powermock.modules.junit4.PowerMockRunner;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Map;

import static org.junit.Assert.fail;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class HmacAuthTest {

    private static final String SIGNED_SIGNATURE = "testEncodedInfo.testSignature";
    private static Subscription hmac_subscription;

    @Mock
    private ISignatureService signatureService;

    @InjectMocks
    private HmacAuth sut;

    @BeforeClass
    public static void setup() {
        setHmac_subscription();
    }

    private static void setHmac_subscription() {
        hmac_subscription = new Subscription();
        hmac_subscription.setName("hamc_test_subscription");
        hmac_subscription.setPushEndpoint("http://challenge");
        hmac_subscription.setDescription("Description");
        hmac_subscription.setTopic("records-changed");
        hmac_subscription.setNotificationId("test-notification-id");
        hmac_subscription.setId("id_1");
        hmac_subscription.setCreatedBy("test@test.com");
        HmacSecret secret = new HmacSecret();
        secret.setValue("testsecret");
        hmac_subscription.setSecret(secret);
    }

    @Test
    public void should_return_valid_URLEncodeQueryString() throws Exception {
        String signedSignatureWithEquals = "someValidBase64=";
        HmacSecret secret = (HmacSecret) hmac_subscription.getSecret();
        when(this.signatureService.getSignedSignature(hmac_subscription.getPushEndpoint(), secret.getValue())).thenReturn(signedSignatureWithEquals);
        sut.setSecret(hmac_subscription.getSecret());
        String pushUrl = sut.getPushUrl(hmac_subscription.getPushEndpoint());
        String expected = hmac_subscription.getPushEndpoint() + "?hmac=" + URLEncoder.encode(signedSignatureWithEquals, StandardCharsets.UTF_8);
        Assert.assertTrue(expected.equalsIgnoreCase(pushUrl));
    }

    @Test
    public void should_return_valid_EndpointAndHeaders() throws Exception {
        HmacSecret secret = (HmacSecret) hmac_subscription.getSecret();
        when(this.signatureService.getSignedSignature(hmac_subscription.getPushEndpoint(), secret.getValue())).thenReturn(SIGNED_SIGNATURE);
        sut.setSecret(hmac_subscription.getSecret());
        Assert.assertEquals(secret, sut.getSecret());
        Map<String, String> headers = sut.getRequestHeaders();
        Assert.assertEquals(0, headers.size());
        String pushUrl = sut.getPushUrl(hmac_subscription.getPushEndpoint());
        Assert.assertEquals(hmac_subscription.getPushEndpoint() + "?hmac=" + SIGNED_SIGNATURE, pushUrl);
    }

    @Test(expected = Exception.class)
    public void should_throwException_whenErrorGeneratingSignature_hmac() throws Exception {
        Exception ex = new Exception("Error generating signed signature");
        HmacSecret secret = (HmacSecret) hmac_subscription.getSecret();
        when(this.signatureService.getSignedSignature(hmac_subscription.getPushEndpoint(), secret.getValue())).thenThrow(ex);
        sut.getPushUrl(hmac_subscription.getPushEndpoint());
        fail("should throw Exception");
    }
}
