// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.queue;

import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.model.Message;
import org.opengroup.osdu.core.aws.sqs.AmazonSQSConfig;
import org.opengroup.osdu.core.aws.ssm.K8sLocalParameterProvider;
import org.opengroup.osdu.core.aws.ssm.K8sParameterNotFoundException;
import org.opengroup.osdu.notification.provider.aws.model.NotificationResult;
import org.opengroup.osdu.notification.provider.aws.model.RetryProcessResult;
import org.opengroup.osdu.notification.provider.aws.queue.impl.NotificationRetryQueueServiceImpl;
import org.opengroup.osdu.notification.provider.aws.utils.SQSUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class NotificationRetrySQSHandler {

    private static final Logger logger = LoggerFactory.getLogger(NotificationRetrySQSHandler.class);
    private static final int MAX_MESSAGE_ALLOWED = 100;
    private static final int MAX_BATCH_REQUEST_COUNT = 10;

    @Value("${aws.region}")
    private String region;

    @Value("${aws.sqs.poll.sleepTime.retryQueue}")
    private Integer sleepTime;

    private String notificationRetryQueueUrl;

    private SQSUtils sqsUtil;

    private NotificationRetryQueueServiceImpl notificationRetryQueueService;

    public NotificationRetrySQSHandler(NotificationRetryQueueServiceImpl notificationRetryQueueService, SQSUtils sqsUtil) {
        this.notificationRetryQueueService = notificationRetryQueueService;
        this.sqsUtil = sqsUtil;
    }

    @Async
    @EventListener(ApplicationStartedEvent.class)
    public void init() throws K8sParameterNotFoundException, InterruptedException {
        K8sLocalParameterProvider provider = new K8sLocalParameterProvider();
        notificationRetryQueueUrl = provider.getParameterAsString("notification-retry-sqs-url");
        processNotifications();
    }

    private void processNotifications() throws InterruptedException {
        List<Message> messages;
        logger.info("Running notification Retry SQS processor");
        while (true) {
            try {
                AmazonSQSConfig sqsConfig = new AmazonSQSConfig(region);
                AmazonSQS sqsClient = sqsConfig.AmazonSQS();
                messages = sqsUtil.getMessages(sqsClient, notificationRetryQueueUrl, MAX_BATCH_REQUEST_COUNT, MAX_MESSAGE_ALLOWED);
                if (messages.isEmpty()) {
                    logger.info("No messages in the retry queue, sleeping for {} seconds", sleepTime);
                    Thread.sleep(sleepTime * 1000L);
                    continue;
                }
                logger.info("Processing {} messages from notification retry queue", messages.size());
                List<RetryProcessResult> notificationResults = notificationRetryQueueService.processNotificationMessages(messages);
                List<Message> failedMessages = notificationResults.stream().filter(msg -> NotificationResult.NACK.equals(msg.getResult())).map(RetryProcessResult::getMessage).collect(
                        Collectors.toList());
                notificationRetryQueueService.changeMessageVisibilityTimeout(sqsClient, notificationRetryQueueUrl, failedMessages);

                List<Message> deleteMessageList = notificationResults.stream().filter(result -> NotificationResult.ACK.equals(result.getResult()))
                        .map(RetryProcessResult::getMessage).collect(Collectors.toList());
                if (!deleteMessageList.isEmpty()) {
                    int deletedCount = sqsUtil.deleteMessages(deleteMessageList, notificationRetryQueueUrl, sqsClient);
                    logger.info("Deleted Messages count :: {}", deletedCount);
                }
            } catch (InterruptedException e) {
                throw e;
            } catch (Exception e) {
                logger.error("General Exception", e);
            }
        }
    }

}