// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.queue;

import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.model.Message;
import org.opengroup.osdu.core.aws.sqs.AmazonSQSConfig;
import org.opengroup.osdu.core.aws.ssm.K8sLocalParameterProvider;
import org.opengroup.osdu.core.aws.ssm.K8sParameterNotFoundException;
import org.opengroup.osdu.notification.provider.aws.utils.SQSUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class NotificationSQSHandler {

    private static final Logger logger = LoggerFactory.getLogger(NotificationSQSHandler.class);
    private static final int MAX_MESSAGE_ALLOWED = 100;
    private static final int MAX_BATCH_REQUEST_COUNT = 10;

    @Value("${aws.region}")
    private String region;

    @Value("${aws.sqs.poll.sleepTime.mainQueue}")
    private Integer sleepTime;
    private String notificationQueueUrl;

    private String notificationDeadLetterQueueUrl;

    private NotificationQueueService notificationQueueService;

    private SQSUtils sqsUtil;

    public NotificationSQSHandler(NotificationQueueService notificationQueueService, SQSUtils sqsUtil) {
        this.notificationQueueService = notificationQueueService;
        this.sqsUtil = sqsUtil;
    }

    @Async
    @EventListener(ApplicationStartedEvent.class)
    public void init() throws K8sParameterNotFoundException, InterruptedException {
        K8sLocalParameterProvider provider = new K8sLocalParameterProvider();
        notificationQueueUrl = provider.getParameterAsString("notification-sqs-url");
        notificationDeadLetterQueueUrl = provider.getParameterAsString("notification-deadletter-queue-url");
        processNotifications();
    }

    private void processNotifications() throws InterruptedException {
        List<Message> messages;
        logger.info("Running notification SQS processor");
        while (true) {
            try {
                AmazonSQSConfig sqsConfig = new AmazonSQSConfig(region);
                AmazonSQS sqsClient = sqsConfig.AmazonSQS();
                messages = sqsUtil.getMessages(sqsClient, notificationQueueUrl, MAX_BATCH_REQUEST_COUNT, MAX_MESSAGE_ALLOWED);
                if (messages.isEmpty()) {
                    logger.info("No messages in the main queue, sleeping for {} seconds", sleepTime);
                    Thread.sleep(sleepTime * 1000L);
                    continue;
                }
                logger.info("Processing {} messages from notification queue", messages.size());

                List<Message> invalidMessages = messages.stream().filter(msg -> !msg.getMessageAttributes().containsKey("data-partition-id")).collect(Collectors.toList());
                List<Message> validMessages = messages.stream().filter(msg -> msg.getMessageAttributes().containsKey("data-partition-id")).collect(Collectors.toList());

                ArrayList<Message> deleteMessageList = new ArrayList<>(invalidMessages);
                ArrayList<Message> deadLetterMessages = new ArrayList<>(invalidMessages);

                if (!deadLetterMessages.isEmpty()) {
                    logger.info("Sending {} invalid messages to dead letter queue", deadLetterMessages.size());
                    sqsUtil.sendMessagesToDeadLetterQueue(notificationDeadLetterQueueUrl, deadLetterMessages, sqsClient);
                }
                List<Message> processedMessages = notificationQueueService.processNotificationMessages(validMessages);
                deleteMessageList.addAll(processedMessages);
                if (!deleteMessageList.isEmpty()) {
                    int deletedCount = sqsUtil.deleteMessages(deleteMessageList, notificationQueueUrl, sqsClient);
                    logger.info("Deleted Messages count :: {}", deletedCount);
                }
            } catch (InterruptedException e) {
                throw e;
            } catch (Exception e) {
                logger.error("General Exception: ", e);
            }
        }
    }

}
