// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.queue.impl;

import com.amazonaws.services.dynamodbv2.datamodeling.PaginatedQueryList;
import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.model.Message;
import com.amazonaws.services.sqs.model.MessageAttributeValue;
import com.amazonaws.services.sqs.model.SendMessageRequest;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelperFactory;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelperV2;
import org.opengroup.osdu.core.aws.sqs.AmazonSQSConfig;
import org.opengroup.osdu.core.aws.ssm.K8sLocalParameterProvider;
import org.opengroup.osdu.core.aws.ssm.K8sParameterNotFoundException;
import org.opengroup.osdu.core.aws.sns.PublishRequestBuilder;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.aws.model.FailedNotificationDoc;
import org.opengroup.osdu.notification.provider.aws.model.NotificationResult;
import org.opengroup.osdu.notification.provider.aws.queue.NotificationQueueService;
import org.opengroup.osdu.notification.provider.aws.repository.SubscriptionRepository;
import org.opengroup.osdu.notification.provider.aws.service.AwsNotificationHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class NotificationQueueServiceImpl implements NotificationQueueService {

    private static final Logger logger = LoggerFactory.getLogger(NotificationQueueServiceImpl.class);


    @Value("${aws.region}")
    private String region;

    @Value("${aws.dynamodb.failedNotificationTable.ssmPath}")
    private String failedNotificationTablePath;

    private SubscriptionRepository subscriptionRepository;

    private DynamoDBQueryHelperFactory dynamoDBQueryHelperFactory;

    private AwsNotificationHandler notificationHandler;

    private AmazonSQS sqsClient;

    private String notificationRetryQueueUrl;


    public NotificationQueueServiceImpl(SubscriptionRepository subscriptionRepository, DynamoDBQueryHelperFactory dynamoDBQueryHelperFactory,
                                        AwsNotificationHandler awsNotificationHandler) {
        this.subscriptionRepository = subscriptionRepository;
        this.dynamoDBQueryHelperFactory = dynamoDBQueryHelperFactory;
        this.notificationHandler = awsNotificationHandler;
    }

    @PostConstruct
    public void init() throws K8sParameterNotFoundException {
        AmazonSQSConfig sqsConfig = new AmazonSQSConfig(region);
        sqsClient = sqsConfig.AmazonSQS();
        K8sLocalParameterProvider provider = new K8sLocalParameterProvider();
        notificationRetryQueueUrl = provider.getParameterAsString("notification-retry-sqs-url");
    }


    @Override
    public List<Message> processNotificationMessages(List<Message> messages) {
        try {
            Map<String, List<Message>> messagesByDataPartition = messages.stream().collect(
                    Collectors.groupingBy(msg -> msg.getMessageAttributes().get("data-partition-id").getStringValue(), HashMap::new, Collectors.toCollection(ArrayList::new)));

            return messagesByDataPartition.entrySet().parallelStream()
                    .map(entry -> processMessagesByDataPartition(entry.getKey(), entry.getValue()))
                    .filter(Objects::nonNull).flatMap(list -> list.stream()).collect(Collectors.toList());
        } catch (Exception e) {
            logger.error("Exception processing messages :", e);
            return new ArrayList<>();
        }
    }

    public List<Message> processMessagesByDataPartition(String dataPartitionId, List<Message> messages) {
        try {
            List<Subscription> subscriptionList = subscriptionRepository.getAllSubscriptionsByDataPartition(dataPartitionId);
            if (!subscriptionList.isEmpty()) {
                subscriptionList.parallelStream().forEach(subscription -> processMessagesBySubscription(subscription, messages, dataPartitionId));
            }
            return messages;
        } catch (Exception e) {
            logger.error("Exception processing messages for dataPartitionId : {}", dataPartitionId, e);
            return new ArrayList<>();
        }
    }

    public void processMessagesBySubscription(Subscription subscription, List<Message> messages, String dataPartitionId) {
        List<Message> messagesToRetry = new ArrayList<>();
        DynamoDBQueryHelperV2 dynamoDBQueryHelper = dynamoDBQueryHelperFactory.getQueryHelperUsingSSM(failedNotificationTablePath);
        try {
            FailedNotificationDoc gsiQuery = new FailedNotificationDoc();
            gsiQuery.setPartitionIdSubscriptionId(String.join("-", dataPartitionId, subscription.getId()));
            PaginatedQueryList<FailedNotificationDoc> results = dynamoDBQueryHelper.queryByGSI(FailedNotificationDoc.class, gsiQuery);

            if (results != null && !results.isEmpty()) {
                logger.debug("Subscription {} has previous failed messages", subscription.getId());
                messagesToRetry.addAll(messages);
            } else {
                boolean hasPreviousFailed = false;
                for (Message message : messages) {
                    logger.debug("Processing Notification for messageId: {}, Subscription Endpoint: {}", message.getMessageId(), subscription.getPushEndpoint());
                    if (hasPreviousFailed) {
                        messagesToRetry.add(message);
                        continue;
                    }
                    Map<String, String> messageAttributes = message.getMessageAttributes().entrySet().stream().collect(
                            Collectors.toMap(Entry::getKey, attribute -> attribute.getValue().getStringValue()));
                    if (!notifySubscriber(subscription, message.getBody(), messageAttributes)) {
                        messagesToRetry.add(message);
                        hasPreviousFailed = true;
                    }
                }
            }
        } catch (Exception e) {
            logger.error("Exception in processMessagesBySubscription for {} :", subscription.getPushEndpoint(), e);
            messagesToRetry.addAll(messages);
        }
        logger.info("Messages retry size :: {}", messagesToRetry.size());
        messagesToRetry.forEach(message -> insertFailedNotification(message, subscription, dataPartitionId, dynamoDBQueryHelper));
    }

    private boolean notifySubscriber(Subscription subscription, String messageBody, Map<String, String> headerAttributes) {
        // Only process this subscription if the topics match.
        if (!subscription.getTopic().equals(headerAttributes.get(PublishRequestBuilder.OSDU_TOPIC_ATTRIBUTE_NAME))) return true;

        HttpResponse response;
        try {
            response = notificationHandler.notifySubscriber(subscription, messageBody, headerAttributes);
        } catch (Exception e) {
            logger.error("Failed to notify subscriber {} : ", subscription.getPushEndpoint(), e);
            return false;
        }
        return response.isSuccessCode();
    }


    private void insertFailedNotification(Message message, Subscription subscription, String dataPartitionId, DynamoDBQueryHelperV2 dynamoDBQueryHelper) {
        logger.debug("Inserting failing notification for :: {}, {}", subscription.getPushEndpoint(), message.getMessageId());
        Long currentTimeMillis = System.currentTimeMillis();
        Long ttl = currentTimeMillis / 1000L + 432000;

        String id = Base64.getEncoder().encodeToString((subscription.getPushEndpoint() + message.getMessageId()).getBytes());
        try {
            Map<String, MessageAttributeValue> messageAttributes = message.getMessageAttributes();

            MessageAttributeValue failedNotificationRecordId = new MessageAttributeValue()
                    .withDataType("String");
            failedNotificationRecordId.setStringValue(id);

            MessageAttributeValue subscriptionIdAttribute = new MessageAttributeValue()
                    .withDataType("String");
            subscriptionIdAttribute.setStringValue(subscription.getId());

            messageAttributes.put("subscriptionId", subscriptionIdAttribute);
            messageAttributes.put("failedNotificationRecordId", failedNotificationRecordId);

            SendMessageRequest sendMessageRequest = new SendMessageRequest()
                    .withQueueUrl(notificationRetryQueueUrl)
                    .withMessageBody(message.getBody())
                    .withMessageAttributes(messageAttributes);
            sqsClient.sendMessage(sendMessageRequest);

            FailedNotificationDoc notificationDoc = FailedNotificationDoc.builder()
                    .id(id)
                    .partitionIdSubscriptionId(String.join("-", dataPartitionId, subscription.getId()))
                    .status(NotificationResult.NACK)
                    .createdOnEpoch(new Timestamp(currentTimeMillis).toString())
                    .dataPartitionId(dataPartitionId)
                    .ttl(ttl).build();
            dynamoDBQueryHelper.saveWithHashCondition(notificationDoc, "id");
        } catch (Exception e) {
            logger.error("Exception while inserting failed record, messageId: {}, subscriptionPushUrl: {} ", message.getMessageId(), subscription.getPushEndpoint(), e);
        }
    }
}