// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.service;

import com.google.common.base.Strings;
import org.opengroup.osdu.core.common.http.HttpClient;
import org.opengroup.osdu.core.common.http.HttpRequest;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.Secret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.auth.factory.AuthFactory;
import org.opengroup.osdu.notification.auth.interfaces.SecretAuth;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.util.Map;

@Service
public class AwsNotificationHandler {

    private static final Logger logger = LoggerFactory.getLogger(AwsNotificationHandler.class);
    private static final String X_COLLABORATION_HEADER = "x-collaboration";

    private HttpClient httpClient;
    private AuthFactory authFactory;
    @Value("${app.waitingTime:30000}")
    private int waitingTime;

    public AwsNotificationHandler(HttpClient httpClient, AuthFactory authFactory) {
        this.httpClient = httpClient;
        this.authFactory = authFactory;
    }

    public HttpResponse notifySubscriber(Subscription subscription, String pubsubMessage, Map<String, String> headerAttributes) throws Exception {
        Secret secret = subscription.getSecret();
        String endpoint = subscription.getPushEndpoint();
        String secretType = secret.getSecretType();
        String collaborationContext = "";

        // Authentication Secret
        SecretAuth secretAuth = authFactory.getSecretAuth(secretType);
        secretAuth.setSecret(secret);
        String pushUrl = secretAuth.getPushUrl(endpoint);
        Map<String, String> requestHeader = secretAuth.getRequestHeaders();

        if (headerAttributes.containsKey(X_COLLABORATION_HEADER)) {
            collaborationContext = headerAttributes.get(X_COLLABORATION_HEADER);
        }

        requestHeader.put(DpsHeaders.CONTENT_TYPE, "application/json");
        requestHeader.put(DpsHeaders.CORRELATION_ID, headerAttributes.get(DpsHeaders.CORRELATION_ID));
        requestHeader.put(DpsHeaders.DATA_PARTITION_ID, headerAttributes.get(DpsHeaders.DATA_PARTITION_ID));

        if (!Strings.isNullOrEmpty(collaborationContext)) {
            requestHeader.put(X_COLLABORATION_HEADER, headerAttributes.get(X_COLLABORATION_HEADER));
        }

        HttpRequest request = HttpRequest.post().url(pushUrl).headers(requestHeader).body(pubsubMessage).connectionTimeout(waitingTime).build();
        logger.debug("Sending out notification to endpoint: {}", endpoint);
        HttpResponse response = httpClient.send(request);

        logger.debug("Request URL : {}, Response Code: {} ,Response Time: {}", subscription.getPushEndpoint(), response.getResponseCode(), Duration.ofMillis(response.getLatency()));

        if (!response.isSuccessCode()) {
            logger.error("Subscriber notification request failed. notificationId={} pushEndpoint={} createdBy={} response={}",
                    subscription.getNotificationId(), subscription.getPushEndpoint(), subscription.getCreatedBy(), response.getBody());
        }
        return response;
    }
}
