// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.aws;


import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;


import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.notification.provider.aws.impl.AwsPubsubRequestBodyExtractor;


import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import java.util.Map;


@RunWith(MockitoJUnitRunner.class)
public class AwsPubsubRequestBodyExtractorTest {

    @Mock
    private DpsHeaders dpsHeaders;

    @Mock
    private JaxRsDpsLog logger;

    @Mock
    private HttpServletRequest request;

    @Mock
    private AwsPubsubRequestBodyExtractor service;

    @Before
    public void init() { }


    @Test
    public void should_returnValidData_extractDataFromRequestBody() throws IOException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t\"correlation-id\": \"39137f49-67d6-4001-a6aa-15521ef4f49e\",\n" +
                "\t\"data-partition-id\": \"" + TestUtils.getDataPartitionId() + "\"\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t},\n" +
                "\t\"subscription\": \""+ "de12345" +"\"\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        String expectedData = "testData";

        service = new AwsPubsubRequestBodyExtractor(request);
        String receivedData = service.extractDataFromRequestBody();

            Assert.assertEquals(expectedData,receivedData);
    }


    @Test
    public void should_returnValidNotificationId_FromRequestBody() throws IOException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t\"correlation-id\": \"39137f49-67d6-4001-a6aa-15521ef4f49e\",\n" +
                "\t\"data-partition-id\": \"" + TestUtils.getDataPartitionId() + "\"\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t},\n" +
                "\t\"subscription\": \""+ "de12345" +"\"\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        String expectedData = "de12345";

        service = new AwsPubsubRequestBodyExtractor(request);
        String receivedData = service.extractNotificationIdFromRequestBody();

        Assert.assertEquals(expectedData,receivedData);
    }
    
    @Test(expected = AppException.class)
    public void extractNotificationIdFromRequestBodyThrowsAppExceptionOnNullSubscription() throws IOException, AppException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t\"correlation-id\": \"39137f49-67d6-4001-a6aa-15521ef4f49e\",\n" +
                "\t\"data-partition-id\": \"" + TestUtils.getDataPartitionId() + "\"\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t}\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        service = new AwsPubsubRequestBodyExtractor(request);
        service.extractNotificationIdFromRequestBody();
    }

    
    @Test(expected = AppException.class)
    public void extractAttributesFromRequestBodyThrowsAppExceptionOnNullAttributes() throws IOException, AppException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t},\n" +
                "\t\"subscription\": \""+ "de12345" +"\"\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        service = new AwsPubsubRequestBodyExtractor(request);
        service.extractAttributesFromRequestBody();
    }
    
    
    @Test(expected = AppException.class)
    public void extractAttributesFromRequestBodyThrowsAppExceptionOnNullDataPartitionId() throws IOException, AppException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t\"correlation-id\": \"39137f49-67d6-4001-a6aa-15521ef4f49e\"\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t},\n" +
                "\t\"subscription\": \""+ "de12345" +"\"\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        service = new AwsPubsubRequestBodyExtractor(request);
        service.extractAttributesFromRequestBody();
    }
    
    @Test
    public void should_returnValidAttributes_FromRequestBody() throws IOException {
        String stringRequest =  "{\n" +
                "\t\"Type\": \"Notification\",\n" +
                "\t\"message\": {\n" +
                "\t\"attributes\": {\n" +
                "\t\"correlation-id\": \"39137f49-123-456\",\n" +
                "\t\"data-partition-id\": \"" + TestUtils.getDataPartitionId() + "\"\n" +
                "\t},\n" +
                "\t\"data\": \"dGVzdERhdGE=\",\n" +
                "\t\"messageId\": \"136969346945\"\n" +
                "\t},\n" +
                "\t\"subscription\": \""+ "de12345" +"\"\n" +
                "}\n";
        BufferedReader reader = new BufferedReader(new StringReader(stringRequest));
        Mockito.when(request.getReader()).thenReturn(reader);

        service = new AwsPubsubRequestBodyExtractor(request);
        // Act
        Map<String, String> receivedAttributes = service.extractAttributesFromRequestBody();

        // Asset
        Assert.assertEquals("39137f49-123-456", receivedAttributes.get("correlation-id"));
        Assert.assertEquals("opendes", receivedAttributes.get("data-partition-id"));
    }
}
