// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.repository;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.withSettings;

import com.amazonaws.services.dynamodbv2.datamodeling.PaginatedQueryList;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.internal.stubbing.defaultanswers.ForwardsInvocations;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelperFactory;
import org.opengroup.osdu.core.aws.dynamodb.DynamoDBQueryHelperV2;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.aws.model.SubscriptionDoc;
import org.opengroup.osdu.notification.provider.aws.security.KmsHelper;
import org.powermock.reflect.Whitebox;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@RunWith(MockitoJUnitRunner.class)
public class SubscriptionRepositoryTest {

    @Mock
    private DynamoDBQueryHelperFactory dynamoDBQueryHelperFactory;

    @Mock
    private KmsHelper kmsHelper;

    @InjectMocks
    private SubscriptionRepository subscriptionRepository;

    @Mock
    private DynamoDBQueryHelperV2 dynamoDBQueryHelper;

    @Before
    public void initTest() {
        Whitebox.setInternalState(subscriptionRepository, "subscriptionTableRelativePath", "services/core/register/SubscriptionTable");

        when(dynamoDBQueryHelperFactory.getQueryHelperForPartition(anyString(), anyString())).thenReturn(dynamoDBQueryHelper);
    }


    @Test
    public void getAllSubscriptions_hmacSecretType_success(){
        SubscriptionDoc subscription1 = new SubscriptionDoc();
        subscription1.setName("My listener");
        subscription1.setTopic("records-changed");
        subscription1.setPushEndpoint("/api/test/subscriber");
        subscription1.setNotificationId("de-859ea6a6-eefa-4e30-ba38-13f8cd71360a");
        subscription1.setId("testSubscriptionId1");
        subscription1.setSecretType("HMAC");
        ArrayList<SubscriptionDoc> results = new ArrayList<>();
        results.add(subscription1);
        when(dynamoDBQueryHelper.queryByGSI(any(), any())).thenReturn(mock(PaginatedQueryList.class, withSettings().defaultAnswer(new ForwardsInvocations(results))));

        List<Subscription> subscriptionList =  subscriptionRepository.getAllSubscriptionsByDataPartition("testDataPartitionId");
        assertEquals(1, subscriptionList.size());
        assertEquals("testSubscriptionId1", subscriptionList.get(0).getId());
    }

    @Test
    public void getAllSubscriptions_gsaSecretType_success(){
        SubscriptionDoc subscription1 = new SubscriptionDoc();
        subscription1.setName("My listener");
        subscription1.setTopic("records-changed");
        subscription1.setPushEndpoint("/api/test/subscriber");
        subscription1.setNotificationId("de-859ea6a6-eefa-4e30-ba38-13f8cd71360a");
        subscription1.setId("testSubscriptionId1");
        subscription1.setSecretType("GSA");
        ArrayList<SubscriptionDoc> results = new ArrayList<>();
        results.add(subscription1);
        when(dynamoDBQueryHelper.queryByGSI(any(), any())).thenReturn(mock(PaginatedQueryList.class, withSettings().defaultAnswer(new ForwardsInvocations(results))));

        when(kmsHelper.decrypt(any(), any())).thenReturn("testSecretValue`test");

        List<Subscription> subscriptionList =  subscriptionRepository.getAllSubscriptionsByDataPartition("testDataPartitionId");
        assertEquals(1, subscriptionList.size());
        assertEquals("testSubscriptionId1", subscriptionList.get(0).getId());
    }

    @Test
    public void getSubscriptionById_hmacSecretType_success(){
        SubscriptionDoc subscription1 = new SubscriptionDoc();
        subscription1.setName("My listener");
        subscription1.setTopic("records-changed");
        subscription1.setPushEndpoint("/api/test/subscriber");
        subscription1.setNotificationId("de-859ea6a6-eefa-4e30-ba38-13f8cd71360a");
        subscription1.setId("testSubscriptionId1");
        subscription1.setSecretType("GSA");
        when(dynamoDBQueryHelper.loadByPrimaryKey(any(), any())).thenReturn(subscription1);
        when(kmsHelper.decrypt(any(), any())).thenReturn("testSecretValue`test");

        Optional<Subscription> subscriptionResponse =  subscriptionRepository.getSubscriptionById("testDataPartitionId", "testSubscriptionId1");
        assertEquals(true, subscriptionResponse.isPresent());
        assertEquals("testSubscriptionId1", subscriptionResponse.get().getId());
    }

    @Test
    public void getSubscriptionById_gsaSecretType_success(){
        SubscriptionDoc subscription1 = new SubscriptionDoc();
        subscription1.setName("My listener");
        subscription1.setTopic("records-changed");
        subscription1.setPushEndpoint("/api/test/subscriber");
        subscription1.setNotificationId("de-859ea6a6-eefa-4e30-ba38-13f8cd71360a");
        subscription1.setId("testSubscriptionId1");
        subscription1.setSecretType("HMAC");
        when(dynamoDBQueryHelper.loadByPrimaryKey(any(), any())).thenReturn(subscription1);

        Optional<Subscription> subscriptionResponse =  subscriptionRepository.getSubscriptionById("testDataPartitionId", "testSubscriptionId1");
        assertEquals(true, subscriptionResponse.isPresent());
        assertEquals("testSubscriptionId1", subscriptionResponse.get().getId());
    }


    @Test
    public void getSubscriptionById_subscriptionNotFound_emptyResponse(){
        when(dynamoDBQueryHelper.loadByPrimaryKey(any(), any())).thenReturn(null);

        Optional<Subscription> subscriptionResponse =  subscriptionRepository.getSubscriptionById("testDataPartitionId", "testSubscriptionId1");
        assertEquals(false, subscriptionResponse.isPresent());
    }
}
