// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.security;

import static org.junit.Assert.assertNotNull;
import static org.mockito.ArgumentMatchers.any;

import java.security.NoSuchAlgorithmException;

import javax.crypto.Mac;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.cryptographic.SignatureServiceException;

import com.google.gson.JsonSyntaxException;

@RunWith(MockitoJUnitRunner.class)
public class ThreadSignatureServiceTest {

    private static final String URL = "http://www.osdu.example.com/signature";
    private static final String SECRET = "93491ab4453ef0306bf08f3703460950";

    private static final String HMAC = "02030405060708.090A123B0C0D0E0F";

    @InjectMocks
    private ThreadSignatureService threadSignatureService;

    @Test
    public void getSignedSignature_success() throws SignatureServiceException {
        String response = threadSignatureService.getSignedSignature(URL, SECRET);
        assertNotNull(response);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignature_nullUrl_signatureServiceExceptio() throws SignatureServiceException {
        threadSignatureService.getSignedSignature(null, SECRET);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignature_nullSecret_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.getSignedSignature(URL, null);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignature_invalidAlgorithm_signatureServiceException() throws SignatureServiceException {
        try (MockedStatic<Mac> mockedMac = Mockito.mockStatic(Mac.class)) {
            mockedMac.when(() -> Mac.getInstance(any())).thenThrow(new NoSuchAlgorithmException());
            threadSignatureService.getSignedSignature(URL, SECRET);
        }
    }

    @Test
    public void getSignedSignatureWithExpiryTime_success() throws SignatureServiceException {
        String response = threadSignatureService.getSignedSignature(URL, SECRET, String.valueOf(System.currentTimeMillis() + 2000), SECRET);
        assertNotNull(response);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignatureWithExpiryTime_nullUrl_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.getSignedSignature(null, SECRET, String.valueOf(System.currentTimeMillis() + 2000), SECRET);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignatureWithExpriyTime_nullSecret_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.getSignedSignature(URL, null, String.valueOf(System.currentTimeMillis() + 2000), SECRET);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignatureWithExpiryTime_expiredSecret_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.getSignedSignature(URL, SECRET, String.valueOf(System.currentTimeMillis() - 100), SECRET);
    }

    @Test(expected = SignatureServiceException.class)
    public void getSignedSignatureWithExpriyTime_invalidAlgorithm_signatureServiceException() throws SignatureServiceException {
        try (MockedStatic<Mac> mockedMac = Mockito.mockStatic(Mac.class)) {
            mockedMac.when(() -> Mac.getInstance(any())).thenThrow(new NoSuchAlgorithmException());
            threadSignatureService.getSignedSignature(URL, SECRET, String.valueOf(System.currentTimeMillis() + 2000), SECRET);
        }
    }

    @Test(expected = SignatureServiceException.class)
    public void verifyHmacSignature_nullHmac_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.verifyHmacSignature(null, SECRET);
    }

    @Test(expected = SignatureServiceException.class)
    public void verifyHmacSignature_nullSecret_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.verifyHmacSignature(HMAC, null);
    }

    @Test(expected = SignatureServiceException.class)
    public void verifyHmacSignature_invalidHmac_signatureServiceException() throws SignatureServiceException {
        threadSignatureService.verifyHmacSignature("invalidHmac", SECRET);
    }

    @Test(expected = JsonSyntaxException.class)
    public void verifyHmacSignature_() throws SignatureServiceException {
        threadSignatureService.verifyHmacSignature(HMAC, SECRET);
    }
}
