// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.service;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.http.HttpClient;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.notification.HmacSecret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.auth.factory.AuthFactory;
import org.opengroup.osdu.notification.auth.interfaces.SecretAuth;
import org.powermock.reflect.Whitebox;

import java.util.HashMap;
import java.util.Map;

@RunWith(MockitoJUnitRunner.class)
public class AwsNotificationHandlerTest {

    @Mock
    private HttpClient httpClient;

    @Mock
    private AuthFactory authFactory;

    @InjectMocks
    private AwsNotificationHandler notificationHandler;

    @Mock
    private SecretAuth secretAuth;

    @Before
    public void beforeTest() {
        Whitebox.setInternalState(notificationHandler, "waitingTime", 100);
    }

    @Test
    public void notifySubscriber_success() throws Exception {
        Subscription subscription = new Subscription();
        subscription.setSecret(new HmacSecret());
        subscription.setPushEndpoint("/api/test");
        Map<String, String> headerAttributes = new HashMap<>();

        when(authFactory.getSecretAuth(any())).thenReturn(secretAuth);
        when(secretAuth.getPushUrl(anyString())).thenReturn("/api/test");

        HttpResponse httpResponse = new HttpResponse();
        httpResponse.setResponseCode(200);
        when(httpClient.send(any())).thenReturn(httpResponse);

        HttpResponse response = notificationHandler.notifySubscriber(subscription, "testMessage", headerAttributes);
        assertEquals(200, response.getResponseCode());
    }

    @Test
    public void notifySubscriber_withCollborationHeader_success() throws Exception {
        Subscription subscription = new Subscription();
        subscription.setSecret(new HmacSecret());
        subscription.setPushEndpoint("/api/test");
        Map<String, String> headerAttributes = new HashMap<>();
        headerAttributes.put("x-collaboration", "a0ad53aa-430d-42b1-b676-2dbac583634c");

        when(authFactory.getSecretAuth(any())).thenReturn(secretAuth);
        when(secretAuth.getPushUrl(anyString())).thenReturn("/api/test");

        HttpResponse httpResponse = new HttpResponse();
        httpResponse.setResponseCode(200);
        when(httpClient.send(any())).thenReturn(httpResponse);

        HttpResponse response = notificationHandler.notifySubscriber(subscription, "testMessage", headerAttributes);
        assertEquals(200, response.getResponseCode());
    }

    @Test
    public void notifySubscriber_failed() throws Exception {
        Subscription subscription = new Subscription();
        subscription.setSecret(new HmacSecret());
        subscription.setPushEndpoint("/api/test");
        Map<String, String> headerAttributes = new HashMap<>();

        when(authFactory.getSecretAuth(any())).thenReturn(secretAuth);
        when(secretAuth.getPushUrl(anyString())).thenReturn("/api/test");

        HttpResponse httpResponse = new HttpResponse();
        httpResponse.setResponseCode(500);
        when(httpClient.send(any())).thenReturn(httpResponse);

        HttpResponse response = notificationHandler.notifySubscriber(subscription, "testMessage", headerAttributes);
        assertEquals(500, response.getResponseCode());
    }

}
