<# This script performs migration of event grid subscriptions to service bus subscriptions for a given configuration file.
   migrationSetup.ps1 is the prerequistes for successful execution of this script.
#>
Param([Parameter(Mandatory = $true)][String]$migrationConfigFilePath)

Add-Type -AssemblyName System.Web

# Generates auth key for cosmos API
Function GenerateAuthorizationSignature
{
    [CmdletBinding()]
    Param
    (
        [Parameter(Mandatory = $true)][String]$method,
        [Parameter(Mandatory = $true)][String]$resourceLink,
        [Parameter(Mandatory = $true)][String]$resourceType,
        [Parameter(Mandatory = $true)][String]$key,
        [Parameter(Mandatory = $true)][String]$keyType,
        [Parameter(Mandatory = $true)][String]$tokenVersion,
        [Parameter(Mandatory = $true)][String]$dateTime
    )
    $hmacSha = New-Object System.Security.Cryptography.HMACSHA256
    $hmacSha.Key = [System.Convert]::FromBase64String($key)

    $payLoad = "$($method.ToLowerInvariant() )`n$($resourceType.ToLowerInvariant() )`n$resourceLink`n$($dateTime.ToLowerInvariant() )`n`n"
    $hashPayLoad = $hmacSha.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($payLoad))
    $signature = [System.Convert]::ToBase64String($hashPayLoad);

    [System.Web.HttpUtility]::UrlEncode("type=$keyType&ver=$tokenVersion&sig=$signature")
}

# Fetch the subscription from cosmos db
Function GetCosmosSubscription
{
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory = $true)][String]$Partition,
        [Parameter(Mandatory = $true)][String]$SubscriptionId,
        [Parameter(Mandatory = $true)][String]$ReadWriteMasterKeyCosmosDb,
        [Parameter(Mandatory = $true)][String]$CosmosDBEndPoint,
        [Parameter(Mandatory = $true)][String]$DatabaseName,
        [Parameter(Mandatory = $true)][String]$ContainerName
    )

    $resourceType = "docs";
    $resourceLink = "dbs/$DatabaseName/colls/$ContainerName/docs/$SubscriptionId"
    $partitionkey = "[""$( $Partition )""]"
    $cosmosURI = "$CosmosDBEndPoint$resourceLink"
    $dateTime = [DateTime]::UtcNow.ToString("r")

    $authToken = GenerateAuthorizationSignature -method "GET" -resourceLink $resourceLink -resourceType $resourceType -key $ReadWriteMasterKeyCosmosDb -keyType "master" -tokenVersion "1.0" -dateTime $dateTime

    $headers = @{ authorization = $authToken; "x-ms-version" = "2017-02-22"; "x-ms-documentdb-partitionkey" = $partitionkey; "x-ms-date" = $dateTime }

    # Call cosmos API to get subscription
    Invoke-RestMethod -Method "GET" -Uri $cosmosURI -headers $headers -Verbose

}

# Update the cosmos subscription
Function UpdateCosmosSubscription
{
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory = $true)][String]$JsonSubscription,
        [Parameter(Mandatory = $true)][String]$Partition,
        [Parameter(Mandatory = $true)][String]$SubscriptionId,
        [Parameter(Mandatory = $true)][String]$ReadWriteMasterKeyCosmosDb,
        [Parameter(Mandatory = $true)][String]$CosmosDBEndPoint,
        [Parameter(Mandatory = $true)][String]$DatabaseName,
        [Parameter(Mandatory = $true)][String]$ContainerName
    )

    $resourceType = "docs";
    $resourceLink = "dbs/$DatabaseName/colls/$ContainerName/docs/$SubscriptionId"
    $partitionkey = "[""$( $Partition )""]"
    $contentType = "application/json"
    $cosmosURI = "$CosmosDBEndPoint$resourceLink"
    $dateTime = [DateTime]::UtcNow.ToString("r")

    $authToken = GenerateAuthorizationSignature -method "PUT" -resourceLink $ResourceLink -resourceType $ResourceType -key $ReadWriteMasterKeyCosmosDb -keyType "master" -tokenVersion "1.0" -dateTime $dateTime

    $headers = @{ authorization = $authToken; "x-ms-version" = "2017-02-22"; "x-ms-documentdb-partitionkey" = $partitionkey; "x-ms-date" = $dateTime }

    # Call cosmos API to update subscription
    Invoke-RestMethod -Method "PUT" -ContentType $contentType -Uri $cosmosURI -headers $headers -Body $JsonSubscription -Verbose

}

# Migration Execution
try
{
    $configurations = Get-Content -Raw -Path $migrationConfigFilePath| ConvertFrom-Json

    foreach ($config in $configurations)
    {
        try
        {
            $ResourceGroupIdCosmos = $config.ResourceGroupIdCosmos
            $PartitionKeyCosmos = $config.PartitionKeyCosmos
            $ReadWriteMasterKeyCosmosDb = $config.ReadWriteMasterKeyCosmosDb
            $CosmosDBEndPoint = $config.CosmosDBEndPoint
            $DatabaseName = $config.DatabaseName
            $ContainerName = $config.ContainerName
            $ResourceGroupNameServiceBus = $config.ResourceGroupNameServiceBus
            $ResourceGroupIdServiceBus = $config.ResourceGroupIdServiceBus
            $NamespaceNameServiceBus = $config.NamespaceNameServiceBus
            $ServiceBusTopicName = $config.ServiceBusTopicName
            $SubscriptionIds = $config.SubscriptionIdsCosmos

            foreach ($SubscriptionId in $SubscriptionIds)
            {
                $Subscription = $null
                $ServiceBusSubscription = $null
                try
                {
                    # Set resource group susbscription for cosmos
                    Select-AzSubscription -SubscriptionName $ResourceGroupIdCosmos -Verbose -ErrorAction Stop

                    $Subscription = GetCosmosSubscription -Partition $PartitionKeyCosmos -SubscriptionId $SubscriptionId -ReadWriteMasterKeyCosmosDb $ReadWriteMasterKeyCosmosDb -CosmosDBEndPoint $CosmosDBEndPoint -DatabaseName $DatabaseName -ContainerName $ContainerName
                    if ((!$Subscription) -or (!$Subscription.notificationId) -or (!$Subscription.topic))
                    {
                        throw "Subscription is not present/null/empty or NotificationId/Topic is not present/null/empty in subscription with ID - $SubscriptionId"
                    }
                    $NotificationId = $Subscription.notificationId

                    # Set resource group susbscription for service bus
                    Select-AzSubscription -SubscriptionName $ResourceGroupIdServiceBus -Verbose -ErrorAction Stop

                    # Create service bus topic subscription
                    $ServiceBusSubscription = New-AzServiceBusSubscription -ResourceGroupName $ResourceGroupNameServiceBus -Namespace $NamespaceNameServiceBus -Topic $ServiceBusTopicName -Name $NotificationId -Verbose

                    if (!$ServiceBusSubscription)
                    {
                        throw "Unable to create service bus subscription with Id - $NotificationId and topic - $ServiceBusTopicName "
                    }
                    # Update subscription topic name
                    $Subscription.topic = $ServiceBusTopicName
                    $JsonSubscription = $Subscription| ConvertTo-Json

                    # Update cosmos subscription
                    $UpdatedSubscription = UpdateCosmosSubscription -JsonSubscription $JsonSubscription -Partition $PartitionKeyCosmos -SubscriptionId $SubscriptionId -ReadWriteMasterKeyCosmosDb $ReadWriteMasterKeyCosmosDb -CosmosDBEndPoint $CosmosDBEndPoint -DatabaseName $DatabaseName -ContainerName $ContainerName

                    Write-Output "Migration to Service Bus is Successful for Subscription - $SubscriptionId `n"
                }
                catch
                {
                    Write-Output "Migration Failed for Subscription - $SubscriptionId." $_
                }
            }
        }
        catch
        {
            Write-Output "Unable to process configuration -$config.ResourceGroupName." $_
        }
    }

}
catch
{
    Write-Output "Unable to process configurations" $_
}