// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.messageBus;

import com.microsoft.azure.servicebus.IMessage;
import com.microsoft.azure.servicebus.Message;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.notification.provider.azure.messageBus.extractor.ServiceBusRequestBodyExtractor;
import org.springframework.http.HttpStatus;

import java.util.Map;

import static org.junit.Assert.fail;

@RunWith(MockitoJUnitRunner.class)
public class ServiceBusRequestBodyExtractorTest {
    @InjectMocks
    private ServiceBusRequestBodyExtractor sut;
    private static final String inValidData = "[{\"invalidData\"}]";
    private static final String validData = "[{\"id\":\"opendes:doc:\",\"kind\":\"opendes:at:wellbore:1.0.0\",\"op\":\"create\"},{\"id\":\"opendes:doc:\",\"kind\":\"opendes:at:wellbore:1.0.0\",\"op\":\"create\"}]";
    private static final String dataPartitionId = "opendes";
    private static final String correlationId = "908fcf8d-30c5-4c74-a0ae-ab47b48b7a85";
    private static final String accountId = "ab47b48b7a85-30c5";
    private static final String collaborationDirectives = "id=9e1c4e74-3b9b-4b17-a0d5-67766558ec65,application=Test App";

    @Test
    public void should_throwWhenAttributesAreMissing_extractDataFromRequestBody() {
        IMessage message = getInvalidMessage();
        try {
            sut.InitializeExtractor(message);
            fail("Should Throw Exception");
        } catch (AppException appException) {
            Assert.assertEquals(HttpStatus.BAD_REQUEST.value(), appException.getError().getCode());
            Assert.assertEquals("Unable to parse request payload.", appException.getError().getMessage());
        } catch (Exception exception) {
            fail("Should Throw AppException");
        }

    }

    @Test
    public void shouldReturnNotificationDataAndAttributesWhenValidRequestBody() {
        IMessage message = getValidMessage();
        try {
            sut.InitializeExtractor(message);
            String notificationData = sut.extractDataFromRequestBody();
            Assert.assertEquals(notificationData, validData);
            Map<String, String> attributes = sut.extractAttributesFromRequestBody();
            Assert.assertEquals(attributes.get("account-id"), accountId);
            Assert.assertEquals(attributes.get("correlation-id"), correlationId);
            Assert.assertEquals(attributes.get("data-partition-id"), dataPartitionId);
            Assert.assertEquals(attributes.get("x-collaboration"), null);

        } catch (Exception exception) {
            fail("Should not Throw AppException");
        }

    }

    @Test
    public void shouldReturnNotificationDataAndAttributesWhenValidRequestBodyWithCollaborationDirectivesProvided() {
        IMessage message = getValidMessageWithCollaborationDirectives();
        try {
            sut.InitializeExtractor(message);
            String notificationData = sut.extractDataFromRequestBody();
            Assert.assertEquals(notificationData, validData);
            Map<String, String> attributes = sut.extractAttributesFromRequestBody();
            Assert.assertEquals(attributes.get("account-id"), accountId);
            Assert.assertEquals(attributes.get("correlation-id"), correlationId);
            Assert.assertEquals(attributes.get("data-partition-id"), dataPartitionId);
            Assert.assertEquals(attributes.get("x-collaboration"), collaborationDirectives);
        } catch (Exception exception) {
            fail("Should not Throw AppException");
        }
    }

    private Message getValidMessageWithCollaborationDirectives() {
        String body =
                "    {\n" +
                        "        \"message\": {\n" +
                        "            \"data\":" + validData + ",\n" +
                        "            \"account-id\": \"" + accountId + "\",\n" +
                        "            \"correlation-id\": \"" + correlationId + "\",\n" +
                        "            \"data-partition-id\": \"" + dataPartitionId + "\",\n" +
                        "            \"x-collaboration\": \"" + collaborationDirectives + "\"\n" +
                        "        }\n" +
                        "    }";
        return new Message(body);
    }


    private Message getValidMessage() {

        String body =
                "    {\n" +
                        "        \"message\": {\n" +
                        "            \"data\":" + validData + ",\n" +
                        "            \"account-id\": \"" + accountId + "\",\n" +
                        "            \"correlation-id\": \"" + correlationId + "\",\n" +
                        "            \"data-partition-id\": \"" + dataPartitionId + "\"\n" +
                        "        }\n" +
                        "    }";
        return new Message(body);
    }

    private Message getInvalidMessage() {
        String body =
                "    {\n" +
                        "        \"message\": {\n" +
                        "            \"data\":" + inValidData + ",\n" +
                        "            \"correlation-id\": \"" + correlationId + "\",\n" +
                        "            \"data-partition-id\": \"" + dataPartitionId + "\"\n" +
                        "        }\n" +
                        "    }";
        Message message = new Message(body);
        return message;
    }
}
