/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.config;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.opengroup.osdu.core.common.cache.IRedisCache;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.gcp.model.ExternalSubscriptions;
import org.opengroup.osdu.notification.provider.gcp.pubsub.OqmSubscriptionHandler;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
@Slf4j
public class ExternalSubscriptionsManager {

  private final IRedisCache<String, ExternalSubscriptions> subscriptionInfoCache;
  private final OqmSubscriptionHandler subscriptionHandler;

  public ExternalSubscriptions getExternalSubscriptions(String dataPartitionId) {
    if (subscriptionInfoCache.get(dataPartitionId) == null) {
      log.debug("Subscription info cache wasn't found for tenant {}", dataPartitionId);
      reloadSubscriptionInfoCache(dataPartitionId);
    }
    return subscriptionInfoCache.get(dataPartitionId);
  }

  public Subscription getSubscription(String dataPartitionId, String subscriptionId, String serviceTopic) {
    List<Subscription> cachedInfos = Optional.ofNullable(getExternalSubscriptions(dataPartitionId))
        .orElseThrow(() -> new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, "Server error", "OQM | Subscription info cache was not initialized"))
        .getSubscriptions();

    Subscription subscription = getFilteredSubscription(cachedInfos, dataPartitionId,
        subscriptionId, serviceTopic);

    if (Objects.nonNull(subscription) && Objects.isNull(subscription.getSecret())) {
      return sendGetSubscriptionRequest(dataPartitionId, subscriptionId, serviceTopic, cachedInfos);
    }
    return subscription;
  }

  private Subscription getFilteredSubscription(List<Subscription> subscriptions, String dataPartitionId,
                                               String subscriptionId, String serviceTopic) {
    List<Subscription> filteredInfos = filterSubscriptionInfosByTopic(subscriptions, subscriptionId, serviceTopic);
    if (filteredInfos.isEmpty()) {
      return sendGetSubscriptionRequest(dataPartitionId, subscriptionId, serviceTopic, subscriptions);
    } else {
      log.debug("Register client cache | `{}` subscriptions info found. The first was taken.", filteredInfos.size());
      return filteredInfos.get(0);
    }
  }

  public void updateExternalSubscriptionsCache(String dataPartitionId, ExternalSubscriptions externalSubscriptions) {
    subscriptionInfoCache.put(dataPartitionId, externalSubscriptions);
    log.debug("Subscription info cache was updated for tenant {} with {} values", dataPartitionId, externalSubscriptions.getSubscriptions().size());
  }

  /**
   * Get all subscription infos from Register service and enrich each of them with secret by additional request
   *
   * @param dataPartitionId partition id
   */
  private void reloadSubscriptionInfoCache(String dataPartitionId) {
    List<Subscription> subscriptionInfos = subscriptionHandler.getAllSubscriptionInfos(dataPartitionId);
    List<Subscription> enrichedSubscriptionInfos = subscriptionInfos.stream()
        .map(subscription -> getFilteredSubscription(subscriptionInfos, dataPartitionId, subscription.getNotificationId(), subscription.getTopic()))
        .collect(Collectors.toList());
    subscriptionInfoCache.put(dataPartitionId, ExternalSubscriptions.builder().subscriptions(enrichedSubscriptionInfos).build());
    log.debug("Subscription info cache PRELOADED for tenant: {}. Size is: {}.", dataPartitionId, enrichedSubscriptionInfos.size());
  }

  private Subscription sendGetSubscriptionRequest(String dataPartitionId, String subscriptionId, String serviceTopic, List<Subscription> cachedInfos) {
    List<Subscription> freshInfos = subscriptionHandler.getSubscriptionsById(dataPartitionId, subscriptionId);
    if (freshInfos.isEmpty()) {
      log.warn("Subscription info with sub ID: `{}` not found", subscriptionId);
      return null;
    }

    List<Subscription> filteredFreshInfos = filterSubscriptionInfosByTopic(freshInfos, subscriptionId, serviceTopic);
    if (filteredFreshInfos.isEmpty()) {
      log.warn("Subscription info with sub ID: `{}` not found", subscriptionId);
      return null;
    }
    cachedInfos.addAll(filteredFreshInfos);
    updateExternalSubscriptionsCache(dataPartitionId, ExternalSubscriptions.builder().subscriptions(cachedInfos).build());

    log.debug("Register client | `{}` subscriptions info found. The first was taken.", filteredFreshInfos.size());
    return filteredFreshInfos.get(0);
  }

  private static List<Subscription> filterSubscriptionInfosByTopic(List<Subscription> infos, String subscriptionId, String serviceTopic) {
    return infos.stream()
        .filter(info -> serviceTopic.equals(info.getTopic()) && subscriptionId.equals(info.getNotificationId()))
        .collect(Collectors.toList());
  }
}