/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.pubsub;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.cache.IRedisCache;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.core.common.provider.interfaces.ITenantFactory;
import org.opengroup.osdu.core.gcp.oqm.driver.OqmDriver;
import org.opengroup.osdu.core.gcp.oqm.model.OqmTopic;
import org.opengroup.osdu.notification.provider.gcp.config.ExternalSubscriptionsManager;
import org.opengroup.osdu.notification.provider.gcp.config.OqmConfigurationProperties;
import org.opengroup.osdu.notification.provider.gcp.model.ExternalSubscriptions;

import java.util.Arrays;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class OqmSubscriberManagerTest {

    @InjectMocks
    private OqmSubscriberManager sut;
    @Mock
    private ITenantFactory tenantFactory;
    @Mock
    private OqmDriver driver;
    @Mock
    private MessageBrokerProvider subscriptionProvider;
    @Mock
    private OqmSubscriptionHandler subscriptionHandler;
    @Mock
    private ExternalSubscriptionsManager externalSubscriptionsManager;

    private List<TenantInfo> tenantInfos;
    private List<OqmTopic> topics;

    @Before
    public void setUp() {
        tenantInfos = createTenants();
        topics = createTopics();
    }

    private List<TenantInfo> createTenants() {
        TenantInfo tenant1 = new TenantInfo();
        tenant1.setId(1L);
        tenant1.setDataPartitionId("tenant1");
        TenantInfo tenant2 = new TenantInfo();
        tenant2.setId(2L);
        tenant2.setDataPartitionId("tenant2");
        TenantInfo tenant3 = new TenantInfo();
        tenant3.setId(3L);
        tenant3.setDataPartitionId("tenant3");
        return Arrays.asList(tenant1, tenant2, tenant3);
    }

    private List<OqmTopic> createTopics() {
        OqmTopic topic1 = OqmTopic.builder().name("topic1").build();
        OqmTopic topic2 = OqmTopic.builder().name("topic2").build();
        OqmTopic topic3 = OqmTopic.builder().name("topic3").build();
        OqmTopic topic4 = OqmTopic.builder().name("topic4").build();
        return Arrays.asList(topic1, topic2,topic3, topic4);
    }

    @Test
    public void testProvisioningControlTopicSubscribers() {
        when(tenantFactory.listTenantInfo()).thenReturn(tenantInfos);
        sut.provisionControlTopicSubscribers();
        verify(driver, times(3)).subscribe(any(), any());
    }

    @Test
    public void testProvisioningServiceTopicSubscribers() {
        when(tenantFactory.listTenantInfo()).thenReturn(tenantInfos);
        when(subscriptionHandler.getTopics(any())).thenReturn(topics);
        sut.provisionServiceSubscribers();
        verify(driver, times(24)).subscribe(any(), any());
    }

    @Test
    public void testProvisioningSubscriptionInfoCache() {
        when(tenantFactory.listTenantInfo()).thenReturn(tenantInfos);
        sut.provisionSubscriptionInfoCache();
        verify(externalSubscriptionsManager, times(3)).getExternalSubscriptions(any());
    }
}