// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.api;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.HmacSecret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.notification.ISubscriptionService;
import org.opengroup.osdu.core.common.notification.SubscriptionAPIConfig;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.core.common.notification.SubscriptionFactory;
import org.opengroup.osdu.notification.util.*;

import com.sun.jersey.api.client.ClientResponse;

import java.util.HashMap;
import java.util.Map;

public class TestPubsubEndpointHMAC extends PubsubEndpointHMACTests {

    private String subscriptionId = null;
    private String notificationId = null;
    private ISubscriptionService awssubscriptionService;
    private static SubscriptionFactory awsfactory;
    @BeforeClass
    public static void classSetup() throws Exception {
        PubsubEndpointHMACTests.classSetup();
        SubscriptionAPIConfig config = SubscriptionAPIConfig.builder().rootUrl(Config.Instance().RegisterServicePath).build();
        awsfactory = new SubscriptionFactory(config);
    }

    @AfterClass
    public static void classTearDown() throws Exception {
    }

    @Before
    @Override
    public void setup() throws Exception {
        this.testUtils = new AwsTestUtils();
    }

    @After
    @Override
    public void tearDown() throws Exception {
        this.testUtils = null;
    }

    @Override
    protected String getArg() {
        return this.notificationId;
    }

    @Test
    @Override
    public void should_return401_when_noAccessOnCustomerTenant() throws Exception {
        ClientResponse response = descriptor.runOnCustomerTenant(getArg(), getOsduTenantAdminCredentials());
        assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
    }

    @Test
    @Override
    public void should_return400_when_makingHttpRequestWithoutToken() throws Exception {
        ClientResponse response =  descriptor.run(getArg(), "");
        assertEquals(error(response.getEntity(String.class)), 401, response.getStatus());
    }


    @Override
    protected void createResource() throws Exception {
        createResourceInPartition(TestUtils.getOsduTenant());
    }

    private void createResourceInPartition(String partitionId) throws Exception{

        Map<String, String> headers = new HashMap<>();
        headers.put(DpsHeaders.DATA_PARTITION_ID, partitionId);
        headers.put(DpsHeaders.AUTHORIZATION, testUtils.getOpsToken());
        //hardcoding user here for 200 response tests. This is just initializing the subscription creation
        headers.put("x-user-id", AwsConfig.getAWSCognitoUser());
        DpsHeaders dpsHeaders = DpsHeaders.createFromMap(headers);
        awssubscriptionService = awsfactory.create(dpsHeaders);

        Map<String,String> h = dpsHeaders.getHeaders();
        System.out.println(h);


        //Create a new subscription to pub/sub
        Subscription subscription = new Subscription();
        subscription.setName("subscription-integration-test-hmac");
        subscription.setDescription("subscription created for hmac integration test");
        subscription.setTopic(Config.Instance().Topic);
        //This seems to be a bug. Dont need to add the string
        //subscription.setPushEndpoint(Config.Instance().HMACPushUrl + "hmac-integration-test");
        subscription.setPushEndpoint(Config.Instance().HMACPushUrl);
        HmacSecret secret = new HmacSecret();
        secret.setValue(Config.Instance().hmacSecretValue);

        subscription.setSecret(secret);
        try {
            Subscription subscriptionCreated = awssubscriptionService.create(subscription);


            notificationId = subscriptionCreated.getNotificationId();
            subscriptionId = subscriptionCreated.getId();
            Config.Instance().NotificationId = notificationId;
        }catch (SubscriptionException e){
            System.out.println("Subscription exception inner response : " + e.getHttpResponse());
            throw e;
        }
    }

    @Override
    protected void deleteResource() throws Exception {
        awssubscriptionService.delete(subscriptionId);
    }

    @Override
    @Test
    public void should_return20X_when_usingCredentialsWithOpsPermission() throws Exception{
        createResource();

        try {
            ClientResponse response = descriptor.run(getArg(), testUtils.getOpsToken());

            assertEquals(error(response.getStatus() == 204 ? "" : response.getEntity(String.class)), expectedOkResponseCode(), response.getStatus());
            assertEquals("DENY", response.getHeaders().getFirst("X-Frame-Options"));
            assertEquals("1; mode=block", response.getHeaders().getFirst("X-XSS-Protection"));
            assertEquals("nosniff", response.getHeaders().getFirst("X-Content-Type-Options"));
            assertEquals("no-cache, no-store, must-revalidate", response.getHeaders().getFirst("Cache-Control"));
            assertEquals("default-src 'self'", response.getHeaders().getFirst("Content-Security-Policy"));
            assertTrue(response.getHeaders().get("Strict-Transport-Security").get(0).contains("max-age=31536000"));
            assertTrue(response.getHeaders().get("Strict-Transport-Security").get(0).contains("includeSubDomains"));
            assertEquals("0", response.getHeaders().getFirst("Expires"));
        } finally {
            deleteResource();
        }
    }

}